#include "mcc_generated_files/mcc.h"
#include "mcc_generated_files/examples/i2c1_master_example.h"
#include "mcc_generated_files/examples/i2c2_master_example.h"

#define TIME_OFF -1         //タイマーを使用しない場合
#define TIME_UP 0           //タイムアップ
#define TIME_SHT35_MAX 100  //ベースタイマカウント値
#define TIME_OUT_MAX    20  //SHT35の通信タイムアウト

#define SLAVE_ADRS 0x45
#define POLYNOMIAL 0x31

#define SLAVE_ADRS_LCD 0x3E//0x3E 0x7C
#define LINE1_ADRS 0x40
#define LINE2_TOP (0x40 +0x80)
#define FUNC1_SET 0x38
#define FUNC2_SET 0x39
#define INT_OSC 0x14
#define CONST_SET 0x77 //0x73
#define PWR_ICON_SET 0x54 //0x56
#define FOLLOWER_SET 0x6C
#define CLR_DISP 0x01
#define DISP_ONOFF_SET 0x0C
#define LCD_TIM_MAX 500

//#define FLOAT_ENABLE

typedef enum{
    SHT35_MEASURE = 0,
    SHT35_WAIT,
    SHT35_READ,
    SHT35_CHK,        
    SHT35_MAX  
}SHT35_MODE;

/* 変数宣言 */
uint8_t singleshot[2] = { 0x2C, 0x06};
int16_t timSht35start;
int8_t  timSht35Out = TIME_OFF;
int16_t timlcd;
SHT35_MODE mode = SHT35_MEASURE;
uint8_t rxdata[6];
uint16_t tempHex;
uint16_t humiHex;
uint8_t chksum[2];
uint8_t disptbl[2][16] ={"temp:   20.00deg","humi:   50.00%  "};

#ifdef FLOAT_ENABLE
float temp;
float humi;
#else
int32_t temp; //100倍値で表現する
uint32_t humi; //100倍値で表現する
#endif

/* 関数ﾌﾟﾛﾄﾀｲﾌﾟ宣言 */
void mainApp(void);
void mainTimer(void);
uint8_t Crc8Calc(uint8_t *data, uint8_t sz );
void LcdInit(void);
void DispSet(void);
void DspClear(void);

/* Main application */
void main(void)
{
    SYSTEM_Initialize();
    TMR0_SetInterruptHandler(mainTimer);
    INTERRUPT_GlobalInterruptEnable();
    INTERRUPT_PeripheralInterruptEnable();
    timSht35start = TIME_SHT35_MAX;
    LcdInit();

    while (1)
    {
        mainApp();
    }
}

/* タイマ管理関数 */
void mainTimer(void){
    
    if( timSht35start > TIME_UP ){
        --timSht35start;
    }
    
    if( timSht35Out > TIME_UP ){
        --timSht35Out;
    }
    
    if( timlcd > TIME_UP ){
        --timlcd;
    }
}
/* メイン処理関数 */
void mainApp(void){
    
    switch(mode){
        case SHT35_MEASURE:
            if( timSht35start == TIME_UP ){
                timSht35start = TIME_OFF;
                timSht35Out = TIME_OUT_MAX;
                I2C2_WriteNBytes(SLAVE_ADRS,&singleshot[0],sizeof(singleshot));
                mode = SHT35_WAIT;
            }
            break;
        case SHT35_WAIT:
            if( RB5_GetValue() == 1){
                mode = SHT35_READ;
            }
            break;
        case SHT35_READ:   
            I2C2_ReadNBytes(SLAVE_ADRS, &rxdata[0], sizeof(rxdata) );
            mode = SHT35_CHK;
            break;
        case SHT35_CHK:
            chksum[0] = Crc8Calc(&rxdata[0],2); //tempのCRCチェック
            chksum[1] = Crc8Calc(&rxdata[3],2); //humiのCRCチェック
            
            if( chksum[0] == rxdata[2] && chksum[1] == rxdata[5]){
                tempHex = ((uint16_t)rxdata[0] << 8) | rxdata[1];
                humiHex =((uint16_t)rxdata[3] << 8) | rxdata[4];

                #ifdef FLOAT_ENABLE
                    temp = (tempHex / 65535.00) * 175 - 45;
                    humi = (humiHex / 65535.0) * 100.0;
                #else
                    //floatの場合処理が重たいので以下で代用
                    //65535でなく65536で割っているため誤差がでるが
                    //処理速度優先のためシフト演算とした。
                    temp = ((uint32_t)tempHex*175*100 >> 16 ) - 45*100;
                    humi = ((uint32_t)humiHex*100*100 >> 16 );
                #endif              
            }
            
            timSht35start = TIME_SHT35_MAX;
            timSht35Out = TIME_OFF;
            mode = SHT35_MEASURE;
            break;
    }
    
    if( timSht35Out == TIME_UP ){
        timSht35Out = TIME_OFF;
        mode = SHT35_MEASURE;
    }
    
    if( timlcd == TIME_UP ){
        timlcd = LCD_TIM_MAX;
        DispSet(); //LCD表示部分をセット
    }     
}
/* CRC8計算関数 */    
uint8_t Crc8Calc(uint8_t *data, uint8_t sz ){
    uint8_t crc = 0xFF;
    uint8_t i,j;
    
    for( i = 0; i < sz; i++){
        crc ^= *data;
        
        for( j = 0; j < 8; j++ ){
            if( crc & 0x80 ){
               crc = ( crc << 1 ) ^ POLYNOMIAL;
            }
            else{
               crc = crc << 1;
            }
        }
        ++data;
    }
    
    return crc;
}
/* LCDの初期化 */            
void LcdInit(void){
    
    __delay_ms(100); //LCDクロックなど安定時間のウェイト
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, FUNC1_SET );
    __delay_us(40);
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, FUNC2_SET );
    __delay_us(40);
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, INT_OSC );
    __delay_us(40);    
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, CONST_SET );
    __delay_us(40);    
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, PWR_ICON_SET );
    __delay_us(40);
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, PWR_ICON_SET );
    __delay_us(40);    
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, FOLLOWER_SET );
    __delay_ms(250);
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, FUNC1_SET );
    __delay_us(40);
    
    DspClear();
    
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, DISP_ONOFF_SET );
    __delay_us(40);
}    
/* LINE2へのカーソル移動処理*/
void DspLine2Top(void){
    
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, LINE2_TOP );
    __delay_us(40);    
}
/* ディスプレイをクリア */
void DspClear(void){
  
    I2C1_Write1ByteRegister(SLAVE_ADRS_LCD, 0x00, CLR_DISP );
    __delay_us(40);
}

/* LCDに表示するデータの生成 */
void DispSet(void){
    uint8_t i;
    bool nega;
    uint8_t disp10000;
    uint8_t disp1000;
    uint8_t disp100;
    uint8_t disp10;
    uint8_t disp1;
    
    DspClear();
    
    if( temp < 0 ){
        temp = abs(temp);
        nega = true;
    }
    
    disp10000 = temp / 10000;
    disp1000 = ( temp % 10000) / 1000; 
    disp100 = (( temp % 10000) % 1000 ) / 100; 
    disp10 = ( temp % 100 ) / 10;
    disp1 = ( temp % 100 ) % 10;
    
    if( disp10000 == 0){
        if( nega == true){
            disptbl[0][7] = '-';
        }
        else{
            disptbl[0][7] = ' ';
        }
    }
    else{
        disptbl[0][7] = (uint8_t)disp10000 + 0x30;
    }
    
    disptbl[0][8] = (uint8_t)disp1000 + 0x30;    
    disptbl[0][9] = (uint8_t)disp100 + 0x30;
    disptbl[0][10] = '.';
    disptbl[0][11] = (uint8_t)disp10 + 0x30;
    disptbl[0][12] = (uint8_t)disp1 + 0x30;    
    
    for( i = 0; i < sizeof(disptbl[0]); i++ ){
        I2C1_Write1ByteRegister(SLAVE_ADRS_LCD,LINE1_ADRS,disptbl[0][i]);
    } 
    
    DspLine2Top();
    
    disp10000 = humi / 10000;
    disp1000 = ( humi % 10000) / 1000; 
    disp100 = (( humi % 10000) % 1000 ) / 100; 
    disp10 = ( humi % 100 ) / 10;
    disp1 = ( humi % 100 ) % 10;   
    
    if( disp10000 == 0){
        disptbl[1][7] = ' ';
    }
    else{
        disptbl[1][7] = (uint8_t)disp10000 + 0x30;
    }
    
    disptbl[1][8] = (uint8_t)disp1000 + 0x30;    
    disptbl[1][9] = (uint8_t)disp100 + 0x30;
    disptbl[1][10] = '.';
    disptbl[1][11] = (uint8_t)disp10 + 0x30;
    disptbl[1][12] = (uint8_t)disp1 + 0x30;      
    
    for( i = 0; i < sizeof(disptbl[1]); i++ ){
        I2C1_Write1ByteRegister(SLAVE_ADRS_LCD,LINE1_ADRS,disptbl[1][i]);
    } 
}
/**
 End of File
*/